@@ -18,6 +18,7 @@
    MA 02110-1301, USA.  */
 
 #include "sysdep.h"
+#include <stdbool.h>
 #include <stdio.h>
 #include <errno.h>
 #include "getopt.h"
@@ -37,446 +38,220 @@
 static const char *program_name = NULL;
 static int debug = 0;
 
-typedef struct initializer
+typedef struct dependency
 {
   const char *name;
-  const char *init;
-} initializer;
+  /* Note: Only direct dependencies should be enumerated.  */
+  const char *deps;
+} dependency;
 
-static initializer cpu_flag_init[] =
+static const dependency isa_dependencies[] =
 {
-  { "CPU_UNKNOWN_FLAGS",
-    "~CpuIAMCU" },
-  { "CPU_GENERIC32_FLAGS",
-    "Cpu186|Cpu286|Cpu386" },
-  { "CPU_GENERIC64_FLAGS",
-    "CPU_PENTIUMPRO_FLAGS|CpuClflush|CpuSYSCALL|CPU_MMX_FLAGS|CPU_SSE2_FLAGS|CpuLM" },
-  { "CPU_NONE_FLAGS",
-   "0" },
-  { "CPU_I186_FLAGS",
-    "Cpu186" },
-  { "CPU_I286_FLAGS",
-    "CPU_I186_FLAGS|Cpu286" },
-  { "CPU_I386_FLAGS",
-    "CPU_I286_FLAGS|Cpu386" },
-  { "CPU_I486_FLAGS",
-    "CPU_I386_FLAGS|Cpu486" },
-  { "CPU_I586_FLAGS",
-    "CPU_I486_FLAGS|Cpu387|Cpu586" },
-  { "CPU_I686_FLAGS",
-    "CPU_I586_FLAGS|Cpu686|Cpu687|CpuCMOV|CpuFXSR" },
-  { "CPU_PENTIUMPRO_FLAGS",
-    "CPU_I686_FLAGS|CpuNop" },
-  { "CPU_P2_FLAGS",
-    "CPU_PENTIUMPRO_FLAGS|CPU_MMX_FLAGS" },
-  { "CPU_P3_FLAGS",
-    "CPU_P2_FLAGS|CPU_SSE_FLAGS" },
-  { "CPU_P4_FLAGS",
-    "CPU_P3_FLAGS|CpuClflush|CPU_SSE2_FLAGS" },
-  { "CPU_NOCONA_FLAGS",
-    "CPU_GENERIC64_FLAGS|CpuFISTTP|CPU_SSE3_FLAGS|CpuCX16" },
-  { "CPU_CORE_FLAGS",
-    "CPU_P4_FLAGS|CpuFISTTP|CPU_SSE3_FLAGS|CpuCX16" },
-  { "CPU_CORE2_FLAGS",
-    "CPU_NOCONA_FLAGS|CPU_SSSE3_FLAGS" },
-  { "CPU_COREI7_FLAGS",
-    "CPU_CORE2_FLAGS|CPU_SSE4_2_FLAGS|CpuRdtscp" },
-  { "CPU_K6_FLAGS",
-    "Cpu186|Cpu286|Cpu386|Cpu486|Cpu586|CpuSYSCALL|Cpu387|CPU_MMX_FLAGS" },
-  { "CPU_K6_2_FLAGS",
-    "CPU_K6_FLAGS|Cpu3dnow" },
-  { "CPU_ATHLON_FLAGS",
-    "CPU_K6_2_FLAGS|Cpu686|Cpu687|CpuNop|Cpu3dnowA" },
-  { "CPU_K8_FLAGS",
-    "CPU_ATHLON_FLAGS|CpuRdtscp|CPU_SSE2_FLAGS|CpuLM" },
-  { "CPU_AMDFAM10_FLAGS",
-    "CPU_K8_FLAGS|CpuFISTTP|CPU_SSE4A_FLAGS|CpuLZCNT|CpuPOPCNT" },
-  { "CPU_BDVER1_FLAGS",
-    "CPU_GENERIC64_FLAGS|CpuFISTTP|CpuRdtscp|CpuCX16|CPU_XOP_FLAGS|CpuLZCNT|CpuPOPCNT|CpuLWP|CpuSVME|CpuAES|CpuPCLMUL|CpuPRFCHW" },
-  { "CPU_BDVER2_FLAGS",
-    "CPU_BDVER1_FLAGS|CpuFMA|CpuBMI|CpuTBM|CpuF16C" },
-  { "CPU_BDVER3_FLAGS",
-    "CPU_BDVER2_FLAGS|CpuXsaveopt|CpuFSGSBase" },
-  { "CPU_BDVER4_FLAGS",
-    "CPU_BDVER3_FLAGS|CpuAVX2|CpuMovbe|CpuBMI2|CpuRdRnd|CpuMWAITX" },
-  { "CPU_ZNVER1_FLAGS",
-    "CPU_GENERIC64_FLAGS|CpuFISTTP|CpuRdtscp|CpuCX16|CPU_AVX2_FLAGS|CpuSSE4A|CpuLZCNT|CpuPOPCNT|CpuSVME|CpuAES|CpuPCLMUL|CpuPRFCHW|CpuFMA|CpuBMI|CpuF16C|CpuXsaveopt|CpuFSGSBase|CpuMovbe|CpuBMI2|CpuRdRnd|CpuADX|CpuRdSeed|CpuSMAP|CpuSHA|CpuXSAVEC|CpuXSAVES|CpuClflushOpt|CpuCLZERO|CpuMWAITX" },
-  { "CPU_ZNVER2_FLAGS",
-    "CPU_ZNVER1_FLAGS|CpuCLWB|CpuRDPID|CpuRDPRU|CpuMCOMMIT|CpuWBNOINVD" },
-  { "CPU_ZNVER3_FLAGS",
-    "CPU_ZNVER2_FLAGS|CpuINVLPGB|CpuTLBSYNC|CpuVAES|CpuVPCLMULQDQ|CpuINVPCID|CpuSNP|CpuOSPKE" },
-  { "CPU_ZNVER4_FLAGS",
-    "CPU_ZNVER3_FLAGS|CpuAVX512F|CpuAVX512DQ|CpuAVX512IFMA|CpuAVX512CD|CpuAVX512BW|CpuAVX512VL|CpuAVX512_BF16|CpuAVX512VBMI|CpuAVX512_VBMI2|CpuAVX512_VNNI|CpuAVX512_BITALG|CpuAVX512_VPOPCNTDQ|CpuGFNI|CpuRMPQUERY" },
-  { "CPU_BTVER1_FLAGS",
-    "CPU_GENERIC64_FLAGS|CpuFISTTP|CpuCX16|CpuRdtscp|CPU_SSSE3_FLAGS|CpuSSE4A|CpuLZCNT|CpuPOPCNT|CpuPRFCHW|CpuCX16|CpuClflush|CpuFISTTP|CpuSVME" },
-  { "CPU_BTVER2_FLAGS",
-    "CPU_BTVER1_FLAGS|CPU_AVX_FLAGS|CpuBMI|CpuF16C|CpuAES|CpuPCLMUL|CpuMovbe|CpuXsaveopt|CpuPRFCHW" },
-  { "CPU_8087_FLAGS",
-    "Cpu8087" },
-  { "CPU_287_FLAGS",
-    "Cpu287" },
-  { "CPU_387_FLAGS",
-    "Cpu387" },
-  { "CPU_687_FLAGS",
-    "CPU_387_FLAGS|Cpu687" },
-  { "CPU_CMOV_FLAGS",
-    "CpuCMOV" },
-  { "CPU_FXSR_FLAGS",
-    "CpuFXSR" },
-  { "CPU_CLFLUSH_FLAGS",
-    "CpuClflush" },
-  { "CPU_NOP_FLAGS",
-    "CpuNop" },
-  { "CPU_SYSCALL_FLAGS",
-    "CpuSYSCALL" },
-  { "CPU_MMX_FLAGS",
-    "CpuMMX" },
-  { "CPU_SSE_FLAGS",
-    "CpuSSE" },
-  { "CPU_SSE2_FLAGS",
-    "CPU_SSE_FLAGS|CpuSSE2" },
-  { "CPU_SSE3_FLAGS",
-    "CPU_SSE2_FLAGS|CpuSSE3" },
-  { "CPU_SSSE3_FLAGS",
-    "CPU_SSE3_FLAGS|CpuSSSE3" },
-  { "CPU_SSE4_1_FLAGS",
-    "CPU_SSSE3_FLAGS|CpuSSE4_1" },
-  { "CPU_SSE4_2_FLAGS",
-    "CPU_SSE4_1_FLAGS|CpuSSE4_2|CpuPOPCNT" },
-  { "CPU_VMX_FLAGS",
-    "CpuVMX" },
-  { "CPU_SMX_FLAGS",
-    "CpuSMX" },
-  { "CPU_XSAVE_FLAGS",
-    "CpuXsave" },
-  { "CPU_XSAVEOPT_FLAGS",
-    "CPU_XSAVE_FLAGS|CpuXsaveopt" },
-  { "CPU_AES_FLAGS",
-    "CPU_SSE2_FLAGS|CpuAES" },
-  { "CPU_PCLMUL_FLAGS",
-    "CPU_SSE2_FLAGS|CpuPCLMUL" },
-  { "CPU_FMA_FLAGS",
-    "CPU_AVX_FLAGS|CpuFMA" },
-  { "CPU_FMA4_FLAGS",
-    "CPU_AVX_FLAGS|CpuFMA4" },
-  { "CPU_XOP_FLAGS",
-    "CPU_SSE4A_FLAGS|CPU_FMA4_FLAGS|CpuXOP" },
-  { "CPU_LWP_FLAGS",
-    "CPU_XSAVE_FLAGS|CpuLWP" },
-  { "CPU_BMI_FLAGS",
-    "CpuBMI" },
-  { "CPU_TBM_FLAGS",
-    "CpuTBM" },
-  { "CPU_MOVBE_FLAGS",
-    "CpuMovbe" },
-  { "CPU_CX16_FLAGS",
-    "CpuCX16" },
-  { "CPU_RDTSCP_FLAGS",
-    "CpuRdtscp" },
-  { "CPU_EPT_FLAGS",
-    "CpuEPT" },
-  { "CPU_FSGSBASE_FLAGS",
-    "CpuFSGSBase" },
-  { "CPU_RDRND_FLAGS",
-    "CpuRdRnd" },
-  { "CPU_F16C_FLAGS",
-    "CPU_AVX_FLAGS|CpuF16C" },
-  { "CPU_BMI2_FLAGS",
-    "CpuBMI2" },
-  { "CPU_LZCNT_FLAGS",
-    "CpuLZCNT" },
-  { "CPU_POPCNT_FLAGS",
-    "CpuPOPCNT" },
-  { "CPU_HLE_FLAGS",
-    "CpuHLE" },
-  { "CPU_RTM_FLAGS",
-    "CpuRTM" },
-  { "CPU_INVPCID_FLAGS",
-    "CpuINVPCID" },
-  { "CPU_VMFUNC_FLAGS",
-    "CpuVMFUNC" },
-  { "CPU_3DNOW_FLAGS",
-    "CPU_MMX_FLAGS|Cpu3dnow" },
-  { "CPU_3DNOWA_FLAGS",
-    "CPU_3DNOW_FLAGS|Cpu3dnowA" },
-  { "CPU_PADLOCK_FLAGS",
-    "CpuPadLock" },
-  { "CPU_SVME_FLAGS",
-    "CpuSVME" },
-  { "CPU_SSE4A_FLAGS",
-    "CPU_SSE3_FLAGS|CpuSSE4a" },
-  { "CPU_ABM_FLAGS",
-    "CpuLZCNT|CpuPOPCNT" },
-  { "CPU_AVX_FLAGS",
-    "CPU_SSE4_2_FLAGS|CPU_XSAVE_FLAGS|CpuAVX" },
-  { "CPU_AVX2_FLAGS",
-    "CPU_AVX_FLAGS|CpuAVX2" },
-  { "CPU_AVX_VNNI_FLAGS",
-    "CPU_AVX2_FLAGS|CpuAVX_VNNI" },
-  { "CPU_AVX512F_FLAGS",
-    "CPU_AVX2_FLAGS|CpuAVX512F" },
-  { "CPU_AVX512CD_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512CD" },
-  { "CPU_AVX512ER_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512ER" },
-  { "CPU_AVX512PF_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512PF" },
-  { "CPU_AVX512DQ_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512DQ" },
-  { "CPU_AVX512BW_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512BW" },
-  { "CPU_AVX512VL_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512VL" },
-  { "CPU_AVX512IFMA_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512IFMA" },
-  { "CPU_AVX512VBMI_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512VBMI" },
-  { "CPU_AVX512_4FMAPS_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512_4FMAPS" },
-  { "CPU_AVX512_4VNNIW_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512_4VNNIW" },
-  { "CPU_AVX512_VPOPCNTDQ_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512_VPOPCNTDQ" },
-  { "CPU_AVX512_VBMI2_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512_VBMI2" },
-  { "CPU_AVX512_VNNI_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512_VNNI" },
-  { "CPU_AVX512_BITALG_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512_BITALG" },
-  { "CPU_AVX512_BF16_FLAGS",
-    "CPU_AVX512F_FLAGS|CpuAVX512_BF16" },
-  { "CPU_AVX512_FP16_FLAGS",
-    "CPU_AVX512BW_FLAGS|CpuAVX512_FP16" },
-  { "CPU_PREFETCHI_FLAGS",
-    "CpuPREFETCHI"},
-  { "CPU_AVX_IFMA_FLAGS",
-    "CPU_AVX2_FLAGS|CpuAVX_IFMA" },
-  { "CPU_AVX_VNNI_INT8_FLAGS",
-    "CPU_AVX2_FLAGS|CpuAVX_VNNI_INT8" },
-  { "CPU_CMPCCXADD_FLAGS",
-    "CpuCMPCCXADD" },
-  { "CPU_WRMSRNS_FLAGS",
-    "CpuWRMSRNS" },
-  { "CPU_MSRLIST_FLAGS",
-    "CpuMSRLIST" },
-  { "CPU_AVX_NE_CONVERT_FLAGS",
-    "CPU_AVX2_FLAGS|CpuAVX_NE_CONVERT" },
-  { "CPU_RAO_INT_FLAGS",
-    "CpuRAO_INT" },
-  { "CPU_IAMCU_FLAGS",
-    "Cpu186|Cpu286|Cpu386|Cpu486|Cpu586|CpuIAMCU" },
-  { "CPU_ADX_FLAGS",
-    "CpuADX" },
-  { "CPU_RDSEED_FLAGS",
-    "CpuRdSeed" },
-  { "CPU_PRFCHW_FLAGS",
-    "CpuPRFCHW" },
-  { "CPU_SMAP_FLAGS",
-    "CpuSMAP" },
-  { "CPU_MPX_FLAGS",
-    "CPU_XSAVE_FLAGS|CpuMPX" },
-  { "CPU_SHA_FLAGS",
-    "CPU_SSE2_FLAGS|CpuSHA" },
-  { "CPU_CLFLUSHOPT_FLAGS",
-    "CpuClflushOpt" },
-  { "CPU_XSAVES_FLAGS",
-    "CPU_XSAVE_FLAGS|CpuXSAVES" },
-  { "CPU_XSAVEC_FLAGS",
-    "CPU_XSAVE_FLAGS|CpuXSAVEC" },
-  { "CPU_PREFETCHWT1_FLAGS",
-    "CpuPREFETCHWT1" },
-  { "CPU_SE1_FLAGS",
-    "CpuSE1" },
-  { "CPU_CLWB_FLAGS",
-    "CpuCLWB" },
-  { "CPU_CLZERO_FLAGS",
-    "CpuCLZERO" },
-  { "CPU_MWAITX_FLAGS",
-    "CpuMWAITX" },
-  { "CPU_OSPKE_FLAGS",
-    "CPU_XSAVE_FLAGS|CpuOSPKE" },
-  { "CPU_RDPID_FLAGS",
-    "CpuRDPID" },
-  { "CPU_PTWRITE_FLAGS",
-    "CpuPTWRITE" },
-  { "CPU_IBT_FLAGS",
-    "CpuIBT" },
-  { "CPU_SHSTK_FLAGS",
-    "CpuSHSTK" },
-  { "CPU_GFNI_FLAGS",
-    "CpuGFNI" },
-  { "CPU_VAES_FLAGS",
-    "CpuVAES" },
-  { "CPU_VPCLMULQDQ_FLAGS",
-    "CpuVPCLMULQDQ" },
-  { "CPU_WBNOINVD_FLAGS",
-    "CpuWBNOINVD" },
-  { "CPU_PCONFIG_FLAGS",
-    "CpuPCONFIG" },
-  { "CPU_WAITPKG_FLAGS",
-    "CpuWAITPKG" },
-  { "CPU_UINTR_FLAGS",
-    "CpuUINTR" },
-  { "CPU_CLDEMOTE_FLAGS",
-    "CpuCLDEMOTE" },
-  { "CPU_AMX_INT8_FLAGS",
-    "CPU_AMX_TILE_FLAGS|CpuAMX_INT8" },
-  { "CPU_AMX_BF16_FLAGS",
-    "CPU_AMX_TILE_FLAGS|CpuAMX_BF16" },
-  { "CPU_AMX_FP16_FLAGS",
-    "CPU_AMX_TILE_FLAGS|CpuAMX_FP16" },
-  { "CPU_AMX_TILE_FLAGS",
-    "CpuAMX_TILE" },
-  { "CPU_MOVDIRI_FLAGS",
-    "CpuMOVDIRI" },
-  { "CPU_MOVDIR64B_FLAGS",
-    "CpuMOVDIR64B" },
-  { "CPU_ENQCMD_FLAGS",
-    "CpuENQCMD" },
-  { "CPU_SERIALIZE_FLAGS",
-    "CpuSERIALIZE" },
-  { "CPU_AVX512_VP2INTERSECT_FLAGS",
-    "CpuAVX512_VP2INTERSECT" },
-  { "CPU_TDX_FLAGS",
-    "CpuTDX" },
-  { "CPU_RDPRU_FLAGS",
-    "CpuRDPRU" },
-  { "CPU_MCOMMIT_FLAGS",
-    "CpuMCOMMIT" },
-  { "CPU_SEV_ES_FLAGS",
-    "CpuSEV_ES" },
-  { "CPU_TSXLDTRK_FLAGS",
-    "CpuTSXLDTRK"},
-  { "CPU_KL_FLAGS",
-    "CpuKL" },
-  { "CPU_WIDEKL_FLAGS",
-    "CpuWideKL" },
-  { "CPU_HRESET_FLAGS",
-    "CpuHRESET"},
-  { "CPU_INVLPGB_FLAGS",
-    "CpuINVLPGB" },
-  { "CPU_TLBSYNC_FLAGS",
-    "CpuTLBSYNC" },
-  { "CPU_SNP_FLAGS",
-    "CpuSNP" },
-  { "CPU_RMPQUERY_FLAGS",
-    "CpuRMPQUERY" },
-  { "CPU_ANY_X87_FLAGS",
-    "CPU_ANY_287_FLAGS|Cpu8087" },
-  { "CPU_ANY_287_FLAGS",
-    "CPU_ANY_387_FLAGS|Cpu287" },
-  { "CPU_ANY_387_FLAGS",
-    "CPU_ANY_687_FLAGS|Cpu387" },
-  { "CPU_ANY_687_FLAGS",
-    "Cpu687|CpuFISTTP" },
-  { "CPU_ANY_CMOV_FLAGS",
-    "CpuCMOV" },
-  { "CPU_ANY_FXSR_FLAGS",
-    "CpuFXSR" },
-  { "CPU_ANY_MMX_FLAGS",
-    "CPU_3DNOWA_FLAGS" },
-  { "CPU_ANY_SSE_FLAGS",
-    "CPU_ANY_SSE2_FLAGS|CpuSSE" },
-  { "CPU_ANY_SSE2_FLAGS",
-    "CPU_ANY_SSE3_FLAGS|CpuSSE2" },
-  { "CPU_ANY_SSE3_FLAGS",
-    "CPU_ANY_SSSE3_FLAGS|CpuSSE3|CpuSSE4a" },
-  { "CPU_ANY_SSSE3_FLAGS",
-    "CPU_ANY_SSE4_1_FLAGS|CpuSSSE3" },
-  { "CPU_ANY_SSE4_1_FLAGS",
-    "CPU_ANY_SSE4_2_FLAGS|CpuSSE4_1" },
-  { "CPU_ANY_SSE4_2_FLAGS",
-    "CpuSSE4_2" },
-  { "CPU_ANY_SSE4A_FLAGS",
-    "CpuSSE4a" },
-  { "CPU_ANY_AVX_FLAGS",
-    "CPU_ANY_AVX2_FLAGS|CpuF16C|CpuFMA|CpuFMA4|CpuXOP|CpuAVX" },
-  { "CPU_ANY_AVX2_FLAGS",
-    "CPU_ANY_AVX512F_FLAGS|CpuAVX2|CpuAVX_VNNI|CpuAVX_IFMA|CpuAVX_VNNI_INT8|CpuAVX_NE_CONVERT" },
-  { "CPU_ANY_AVX512F_FLAGS",
-    "CpuAVX512F|CpuAVX512CD|CpuAVX512ER|CpuAVX512PF|CpuAVX512DQ|CPU_ANY_AVX512BW_FLAGS|CpuAVX512VL|CpuAVX512IFMA|CpuAVX512VBMI|CpuAVX512_4FMAPS|CpuAVX512_4VNNIW|CpuAVX512_VPOPCNTDQ|CpuAVX512_VBMI2|CpuAVX512_VNNI|CpuAVX512_BITALG|CpuAVX512_BF16|CpuAVX512_VP2INTERSECT" },
-  { "CPU_ANY_AVX512CD_FLAGS",
-    "CpuAVX512CD" },
-  { "CPU_ANY_AVX512ER_FLAGS",
-    "CpuAVX512ER" },
-  { "CPU_ANY_AVX512PF_FLAGS",
-    "CpuAVX512PF" },
-  { "CPU_ANY_AVX512DQ_FLAGS",
-    "CpuAVX512DQ" },
-  { "CPU_ANY_AVX512BW_FLAGS",
-    "CpuAVX512BW|CPU_ANY_AVX512_FP16_FLAGS" },
-  { "CPU_ANY_AVX512VL_FLAGS",
-    "CpuAVX512VL" },
-  { "CPU_ANY_AVX512IFMA_FLAGS",
-    "CpuAVX512IFMA" },
-  { "CPU_ANY_AVX512VBMI_FLAGS",
-    "CpuAVX512VBMI" },
-  { "CPU_ANY_AVX512_4FMAPS_FLAGS",
-    "CpuAVX512_4FMAPS" },
-  { "CPU_ANY_AVX512_4VNNIW_FLAGS",
-    "CpuAVX512_4VNNIW" },
-  { "CPU_ANY_AVX512_VPOPCNTDQ_FLAGS",
-    "CpuAVX512_VPOPCNTDQ" },
-  { "CPU_ANY_IBT_FLAGS",
-    "CpuIBT" },
-  { "CPU_ANY_SHSTK_FLAGS",
-    "CpuSHSTK" },
-  { "CPU_ANY_AVX512_VBMI2_FLAGS",
-    "CpuAVX512_VBMI2" },
-  { "CPU_ANY_AVX512_VNNI_FLAGS",
-    "CpuAVX512_VNNI" },
-  { "CPU_ANY_AVX512_BITALG_FLAGS",
-    "CpuAVX512_BITALG" },
-  { "CPU_ANY_AVX512_BF16_FLAGS",
-    "CpuAVX512_BF16" },
-  { "CPU_ANY_AMX_INT8_FLAGS",
-    "CpuAMX_INT8" },
-  { "CPU_ANY_AMX_BF16_FLAGS",
-    "CpuAMX_BF16" },
-  { "CPU_ANY_AMX_TILE_FLAGS",
-    "CpuAMX_TILE|CpuAMX_INT8|CpuAMX_BF16|CpuAMX_FP16" },
-  { "CPU_ANY_AVX_VNNI_FLAGS",
-    "CpuAVX_VNNI" },
-  { "CPU_ANY_MOVDIRI_FLAGS",
-    "CpuMOVDIRI" },
-  { "CPU_ANY_UINTR_FLAGS",
-    "CpuUINTR" },
-  { "CPU_ANY_MOVDIR64B_FLAGS",
-    "CpuMOVDIR64B" },
-  { "CPU_ANY_ENQCMD_FLAGS",
-    "CpuENQCMD" },
-  { "CPU_ANY_SERIALIZE_FLAGS",
-    "CpuSERIALIZE" },
-  { "CPU_ANY_AVX512_VP2INTERSECT_FLAGS",
-    "CpuAVX512_VP2INTERSECT" },
-  { "CPU_ANY_TDX_FLAGS",
-    "CpuTDX" },
-  { "CPU_ANY_TSXLDTRK_FLAGS",
-    "CpuTSXLDTRK" },
-  { "CPU_ANY_KL_FLAGS",
-    "CpuKL|CpuWideKL" },
-  { "CPU_ANY_WIDEKL_FLAGS",
-    "CpuWideKL" },
-  { "CPU_ANY_HRESET_FLAGS",
-    "CpuHRESET" },
-  { "CPU_ANY_AVX512_FP16_FLAGS",
-    "CpuAVX512_FP16" },
-  { "CPU_ANY_AVX_IFMA_FLAGS",
-    "CpuAVX_IFMA" },
-  { "CPU_ANY_AVX_VNNI_INT8_FLAGS",
-    "CpuAVX_VNNI_INT8" },
-  { "CPU_ANY_CMPCCXADD_FLAGS",
-    "CpuCMPCCXADD" },
-  { "CPU_ANY_WRMSRNS_FLAGS",
-    "CpuWRMSRNS" },
-  { "CPU_ANY_MSRLIST_FLAGS",
-    "CpuMSRLIST" },
-  { "CPU_ANY_AVX_NE_CONVERT_FLAGS",
-    "CpuAVX_NE_CONVERT" },
-  { "CPU_ANY_RAO_INT_FLAGS",
-    "CpuRAO_INT"},
+  { "UNKNOWN",
+    "~IAMCU" },
+  { "GENERIC32",
+    "386" },
+  { "GENERIC64",
+    "PENTIUMPRO|Clflush|SYSCALL|MMX|SSE2|LM" },
+  { "NONE",
+    "0" },
+  { "PENTIUMPRO",
+    "686|Nop" },
+  { "P2",
+    "PENTIUMPRO|MMX" },
+  { "P3",
+    "P2|SSE" },
+  { "P4",
+    "P3|Clflush|SSE2" },
+  { "NOCONA",
+    "GENERIC64|FISTTP|SSE3|CX16" },
+  { "CORE",
+    "P4|FISTTP|SSE3|CX16" },
+  { "CORE2",
+    "NOCONA|SSSE3" },
+  { "COREI7",
+    "CORE2|SSE4_2|Rdtscp" },
+  { "K6",
+    "186|286|386|486|586|SYSCALL|387|MMX" },
+  { "K6_2",
+    "K6|3dnow" },
+  { "ATHLON",
+    "K6_2|686:min|687|Nop|3dnowA" },
+  { "K8",
+    "ATHLON|Rdtscp|SSE2|LM" },
+  { "AMDFAM10",
+    "K8|FISTTP|SSE4A|ABM" },
+  { "BDVER1",
+    "GENERIC64|FISTTP|Rdtscp|CX16|XOP|ABM|LWP|SVME|AES|PCLMUL|PRFCHW" },
+  { "BDVER2",
+    "BDVER1|FMA|BMI|TBM|F16C" },
+  { "BDVER3",
+    "BDVER2|Xsaveopt|FSGSBase" },
+  { "BDVER4",
+    "BDVER3|AVX2|Movbe|BMI2|RdRnd|MWAITX" },
+  { "ZNVER1",
+    "GENERIC64|FISTTP|Rdtscp|CX16|AVX2|SSE4A|ABM|SVME|AES|PCLMUL|PRFCHW|FMA|BMI|F16C|Xsaveopt|FSGSBase|Movbe|BMI2|RdRnd|ADX|RdSeed|SMAP|SHA|XSAVEC|XSAVES|ClflushOpt|CLZERO|MWAITX" },
+  { "ZNVER2",
+    "ZNVER1|CLWB|RDPID|RDPRU|MCOMMIT|WBNOINVD" },
+  { "ZNVER3",
+    "ZNVER2|INVLPGB|TLBSYNC|VAES|VPCLMULQDQ|INVPCID|SNP|OSPKE" },
+  { "ZNVER4",
+    "ZNVER3|AVX512F|AVX512DQ|AVX512IFMA|AVX512CD|AVX512BW|AVX512VL|AVX512_BF16|AVX512VBMI|AVX512_VBMI2|AVX512_VNNI|AVX512_BITALG|AVX512_VPOPCNTDQ|GFNI|RMPQUERY" },
+  { "BTVER1",
+    "GENERIC64|FISTTP|CX16|Rdtscp|SSSE3|SSE4A|ABM|PRFCHW|CX16|Clflush|FISTTP|SVME" },
+  { "BTVER2",
+    "BTVER1|AVX|BMI|F16C|AES|PCLMUL|Movbe|Xsaveopt|PRFCHW" },
+  { "286",
+    "186" },
+  { "386",
+    "286" },
+  { "486",
+    "386" },
+  { "586",
+    "486|387" },
+  { "586:nofpu",
+    "486" },
+  { "686",
+    "586|687|CMOV|FXSR" },
+  { "686:min",
+    "586|687" },
+  { "687",
+    "387" },
+  { "FISTTP",
+    "687" },
+  { "SSE",
+    "FXSR" },
+  { "SSE2",
+    "SSE" },
+  { "SSE3",
+    "SSE2" },
+  { "SSSE3",
+    "SSE3" },
+  { "SSE4_1",
+    "SSSE3" },
+  { "SSE4_2",
+    "SSE4_1|POPCNT" },
+  { "Xsaveopt",
+    "XSAVE" },
+  { "AES",
+    "SSE2" },
+  { "PCLMUL",
+    "SSE2" },
+  { "FMA",
+    "AVX" },
+  { "FMA4",
+    "AVX" },
+  { "XOP",
+    "SSE4A|FMA4" },
+  { "LWP",
+    "XSAVE" },
+  { "F16C",
+    "AVX" },
+  { "3dnow",
+    "MMX" },
+  { "3dnowA",
+    "3dnow" },
+  { "SSE4a",
+    "SSE3" },
+  { "ABM",
+    "LZCNT|POPCNT" },
+  { "AVX",
+    "SSE4_2|XSAVE" },
+  { "AVX2",
+    "AVX" },
+  { "AVX_VNNI",
+    "AVX2" },
+  { "AVX_IFMA",
+    "AVX2" },
+  { "AVX_VNNI_INT8",
+    "AVX2" },
+  { "AVX_NE_CONVERT",
+    "AVX2" },
+  { "AVX512F",
+    "AVX2" },
+  { "AVX512CD",
+    "AVX512F" },
+  { "AVX512ER",
+    "AVX512F" },
+  { "AVX512PF",
+    "AVX512F" },
+  { "AVX512DQ",
+    "AVX512F" },
+  { "AVX512BW",
+    "AVX512F" },
+  { "AVX512VL",
+    "AVX512F" },
+  { "AVX512IFMA",
+    "AVX512F" },
+  { "AVX512VBMI",
+    "AVX512BW" },
+  { "AVX512_4FMAPS",
+    "AVX512F" },
+  { "AVX512_4VNNIW",
+    "AVX512F" },
+  { "AVX512_VPOPCNTDQ",
+    "AVX512F" },
+  { "AVX512_VBMI2",
+    "AVX512BW" },
+  { "AVX512_VNNI",
+    "AVX512F" },
+  { "AVX512_BITALG",
+    "AVX512BW" },
+  { "AVX512_VP2INTERSECT",
+    "AVX512F" },
+  { "AVX512_BF16",
+    "AVX512BW" },
+  { "AVX512_FP16",
+    "AVX512BW" },
+  { "IAMCU",
+    "586:nofpu" },
+  { "EPT",
+    "VMX" },
+  { "VMFUNC",
+    "VMX" },
+  { "MPX",
+    "XSAVE" },
+  { "SHA",
+    "SSE2" },
+  { "XSAVES",
+    "XSAVEC" },
+  { "XSAVEC",
+    "XSAVE" },
+  { "OSPKE",
+    "XSAVE" },
+  { "GFNI",
+    "SSE2" },
+  { "VAES",
+    "AVX2" },
+  { "VPCLMULQDQ",
+    "AVX2" },
+  { "SEV_ES",
+    "SVME" },
+  { "SNP",
+    "SEV_ES" },
+  { "RMPQUERY",
+    "SNP" },
+  { "TSX",
+    "RTM|HLE" },
+  { "TSXLDTRK",
+    "RTM" },
+  { "AMX_TILE",
+    "XSAVE" },
+  { "AMX_INT8",
+    "AMX_TILE" },
+  { "AMX_BF16",
+    "AMX_TILE" },
+  { "AMX_FP16",
+    "AMX_TILE" },
+  { "KL",
+    "SSE2" },
+  { "WIDEKL",
+    "KL" },
 };
 
+/* This array is populated as process_i386_initializers() walks cpu_flags[].  */
+static unsigned char isa_reverse_deps[Cpu64][Cpu64];
+
 typedef struct bitfield
 {
   int position;
@@ -484,149 +259,152 @@ typedef struct bitfield
   const char *name;
 } bitfield;
 
-#define BITFIELD(n) { n, 0, #n }
+#define BITFIELD(n) { Cpu##n, 0, #n }
 
 static bitfield cpu_flags[] =
 {
-  BITFIELD (Cpu186),
-  BITFIELD (Cpu286),
-  BITFIELD (Cpu386),
-  BITFIELD (Cpu486),
-  BITFIELD (Cpu586),
-  BITFIELD (Cpu686),
-  BITFIELD (CpuCMOV),
-  BITFIELD (CpuFXSR),
-  BITFIELD (CpuClflush),
-  BITFIELD (CpuNop),
-  BITFIELD (CpuSYSCALL),
-  BITFIELD (Cpu8087),
-  BITFIELD (Cpu287),
-  BITFIELD (Cpu387),
-  BITFIELD (Cpu687),
-  BITFIELD (CpuFISTTP),
-  BITFIELD (CpuMMX),
-  BITFIELD (CpuSSE),
-  BITFIELD (CpuSSE2),
-  BITFIELD (CpuSSE3),
-  BITFIELD (CpuSSSE3),
-  BITFIELD (CpuSSE4_1),
-  BITFIELD (CpuSSE4_2),
-  BITFIELD (CpuAVX),
-  BITFIELD (CpuAVX2),
-  BITFIELD (CpuAVX512F),
-  BITFIELD (CpuAVX512CD),
-  BITFIELD (CpuAVX512ER),
-  BITFIELD (CpuAVX512PF),
-  BITFIELD (CpuAVX512VL),
-  BITFIELD (CpuAVX512DQ),
-  BITFIELD (CpuAVX512BW),
-  BITFIELD (CpuIAMCU),
-  BITFIELD (CpuSSE4a),
-  BITFIELD (Cpu3dnow),
-  BITFIELD (Cpu3dnowA),
-  BITFIELD (CpuPadLock),
-  BITFIELD (CpuSVME),
-  BITFIELD (CpuVMX),
-  BITFIELD (CpuSMX),
-  BITFIELD (CpuXsave),
-  BITFIELD (CpuXsaveopt),
-  BITFIELD (CpuAES),
-  BITFIELD (CpuPCLMUL),
-  BITFIELD (CpuFMA),
-  BITFIELD (CpuFMA4),
-  BITFIELD (CpuXOP),
-  BITFIELD (CpuLWP),
-  BITFIELD (CpuBMI),
-  BITFIELD (CpuTBM),
-  BITFIELD (CpuLM),
-  BITFIELD (CpuMovbe),
-  BITFIELD (CpuCX16),
-  BITFIELD (CpuEPT),
-  BITFIELD (CpuRdtscp),
-  BITFIELD (CpuFSGSBase),
-  BITFIELD (CpuRdRnd),
-  BITFIELD (CpuF16C),
-  BITFIELD (CpuBMI2),
-  BITFIELD (CpuLZCNT),
-  BITFIELD (CpuPOPCNT),
-  BITFIELD (CpuHLE),
-  BITFIELD (CpuRTM),
-  BITFIELD (CpuINVPCID),
-  BITFIELD (CpuVMFUNC),
-  BITFIELD (CpuRDSEED),
-  BITFIELD (CpuADX),
-  BITFIELD (CpuPRFCHW),
-  BITFIELD (CpuSMAP),
-  BITFIELD (CpuSHA),
-  BITFIELD (CpuClflushOpt),
-  BITFIELD (CpuXSAVES),
-  BITFIELD (CpuXSAVEC),
-  BITFIELD (CpuPREFETCHWT1),
-  BITFIELD (CpuSE1),
-  BITFIELD (CpuCLWB),
-  BITFIELD (CpuMPX),
-  BITFIELD (CpuAVX512IFMA),
-  BITFIELD (CpuAVX512VBMI),
-  BITFIELD (CpuAVX512_4FMAPS),
-  BITFIELD (CpuAVX512_4VNNIW),
-  BITFIELD (CpuAVX512_VPOPCNTDQ),
-  BITFIELD (CpuAVX512_VBMI2),
-  BITFIELD (CpuAVX512_VNNI),
-  BITFIELD (CpuAVX512_BITALG),
-  BITFIELD (CpuAVX512_BF16),
-  BITFIELD (CpuAVX512_VP2INTERSECT),
-  BITFIELD (CpuTDX),
-  BITFIELD (CpuAVX_VNNI),
-  BITFIELD (CpuAVX512_FP16),
-  BITFIELD (CpuPREFETCHI),
-  BITFIELD (CpuAVX_IFMA),
-  BITFIELD (CpuAVX_VNNI_INT8),
-  BITFIELD (CpuCMPCCXADD),
-  BITFIELD (CpuWRMSRNS),
-  BITFIELD (CpuMSRLIST),
-  BITFIELD (CpuAVX_NE_CONVERT),
-  BITFIELD (CpuRAO_INT),
-  BITFIELD (CpuMWAITX),
-  BITFIELD (CpuCLZERO),
-  BITFIELD (CpuOSPKE),
-  BITFIELD (CpuRDPID),
-  BITFIELD (CpuPTWRITE),
-  BITFIELD (CpuIBT),
-  BITFIELD (CpuSHSTK),
-  BITFIELD (CpuGFNI),
-  BITFIELD (CpuVAES),
-  BITFIELD (CpuVPCLMULQDQ),
-  BITFIELD (CpuWBNOINVD),
-  BITFIELD (CpuPCONFIG),
-  BITFIELD (CpuWAITPKG),
-  BITFIELD (CpuUINTR),
-  BITFIELD (CpuCLDEMOTE),
-  BITFIELD (CpuAMX_INT8),
-  BITFIELD (CpuAMX_BF16),
-  BITFIELD (CpuAMX_FP16),
-  BITFIELD (CpuAMX_TILE),
-  BITFIELD (CpuMOVDIRI),
-  BITFIELD (CpuMOVDIR64B),
-  BITFIELD (CpuENQCMD),
-  BITFIELD (CpuSERIALIZE),
-  BITFIELD (CpuRDPRU),
-  BITFIELD (CpuMCOMMIT),
-  BITFIELD (CpuSEV_ES),
-  BITFIELD (CpuTSXLDTRK),
-  BITFIELD (CpuKL),
-  BITFIELD (CpuWideKL),
-  BITFIELD (CpuHRESET),
-  BITFIELD (CpuINVLPGB),
-  BITFIELD (CpuTLBSYNC),
-  BITFIELD (CpuSNP),
-  BITFIELD (CpuRMPQUERY),
-  BITFIELD (Cpu64),
-  BITFIELD (CpuNo64),
+  BITFIELD (186),
+  BITFIELD (286),
+  BITFIELD (386),
+  BITFIELD (486),
+  BITFIELD (586),
+  BITFIELD (686),
+  BITFIELD (CMOV),
+  BITFIELD (FXSR),
+  BITFIELD (Clflush),
+  BITFIELD (Nop),
+  BITFIELD (SYSCALL),
+  BITFIELD (8087),
+  BITFIELD (287),
+  BITFIELD (387),
+  BITFIELD (687),
+  BITFIELD (FISTTP),
+  BITFIELD (MMX),
+  BITFIELD (SSE),
+  BITFIELD (SSE2),
+  BITFIELD (SSE3),
+  BITFIELD (SSSE3),
+  BITFIELD (SSE4_1),
+  BITFIELD (SSE4_2),
+  BITFIELD (AVX),
+  BITFIELD (AVX2),
+  BITFIELD (AVX512F),
+  BITFIELD (AVX512CD),
+  BITFIELD (AVX512ER),
+  BITFIELD (AVX512PF),
+  BITFIELD (AVX512VL),
+  BITFIELD (AVX512DQ),
+  BITFIELD (AVX512BW),
+  BITFIELD (IAMCU),
+  BITFIELD (SSE4a),
+  BITFIELD (3dnow),
+  BITFIELD (3dnowA),
+  BITFIELD (PadLock),
+  BITFIELD (SVME),
+  BITFIELD (VMX),
+  BITFIELD (SMX),
+  BITFIELD (Xsave),
+  BITFIELD (Xsaveopt),
+  BITFIELD (AES),
+  BITFIELD (PCLMUL),
+  BITFIELD (FMA),
+  BITFIELD (FMA4),
+  BITFIELD (XOP),
+  BITFIELD (LWP),
+  BITFIELD (BMI),
+  BITFIELD (TBM),
+  BITFIELD (LM),
+  BITFIELD (Movbe),
+  BITFIELD (CX16),
+  BITFIELD (EPT),
+  BITFIELD (Rdtscp),
+  BITFIELD (FSGSBase),
+  BITFIELD (RdRnd),
+  BITFIELD (F16C),
+  BITFIELD (BMI2),
+  BITFIELD (LZCNT),
+  BITFIELD (POPCNT),
+  BITFIELD (HLE),
+  BITFIELD (RTM),
+  BITFIELD (INVPCID),
+  BITFIELD (VMFUNC),
+  BITFIELD (RDSEED),
+  BITFIELD (ADX),
+  BITFIELD (PRFCHW),
+  BITFIELD (SMAP),
+  BITFIELD (SHA),
+  BITFIELD (ClflushOpt),
+  BITFIELD (XSAVES),
+  BITFIELD (XSAVEC),
+  BITFIELD (PREFETCHWT1),
+  BITFIELD (SE1),
+  BITFIELD (CLWB),
+  BITFIELD (MPX),
+  BITFIELD (AVX512IFMA),
+  BITFIELD (AVX512VBMI),
+  BITFIELD (AVX512_4FMAPS),
+  BITFIELD (AVX512_4VNNIW),
+  BITFIELD (AVX512_VPOPCNTDQ),
+  BITFIELD (AVX512_VBMI2),
+  BITFIELD (AVX512_VNNI),
+  BITFIELD (AVX512_BITALG),
+  BITFIELD (AVX512_BF16),
+  BITFIELD (AVX512_VP2INTERSECT),
+  BITFIELD (TDX),
+  BITFIELD (AVX_VNNI),
+  BITFIELD (AVX512_FP16),
+  BITFIELD (PREFETCHI),
+  BITFIELD (AVX_IFMA),
+  BITFIELD (AVX_VNNI_INT8),
+  BITFIELD (CMPCCXADD),
+  BITFIELD (WRMSRNS),
+  BITFIELD (MSRLIST),
+  BITFIELD (AVX_NE_CONVERT),
+  BITFIELD (RAO_INT),
+  BITFIELD (MWAITX),
+  BITFIELD (CLZERO),
+  BITFIELD (OSPKE),
+  BITFIELD (RDPID),
+  BITFIELD (PTWRITE),
+  BITFIELD (IBT),
+  BITFIELD (SHSTK),
+  BITFIELD (GFNI),
+  BITFIELD (VAES),
+  BITFIELD (VPCLMULQDQ),
+  BITFIELD (WBNOINVD),
+  BITFIELD (PCONFIG),
+  BITFIELD (WAITPKG),
+  BITFIELD (UINTR),
+  BITFIELD (CLDEMOTE),
+  BITFIELD (AMX_INT8),
+  BITFIELD (AMX_BF16),
+  BITFIELD (AMX_FP16),
+  BITFIELD (AMX_TILE),
+  BITFIELD (MOVDIRI),
+  BITFIELD (MOVDIR64B),
+  BITFIELD (ENQCMD),
+  BITFIELD (SERIALIZE),
+  BITFIELD (RDPRU),
+  BITFIELD (MCOMMIT),
+  BITFIELD (SEV_ES),
+  BITFIELD (TSXLDTRK),
+  BITFIELD (KL),
+  BITFIELD (WideKL),
+  BITFIELD (HRESET),
+  BITFIELD (INVLPGB),
+  BITFIELD (TLBSYNC),
+  BITFIELD (SNP),
+  BITFIELD (RMPQUERY),
+  BITFIELD (64),
+  BITFIELD (No64),
 #ifdef CpuUnused
-  BITFIELD (CpuUnused),
+  BITFIELD (Unused),
 #endif
 };
 
+#undef BITFIELD
+#define BITFIELD(n) { n, 0, #n }
+
 static bitfield opcode_modifiers[] =
 {
   BITFIELD (D),
@@ -636,7 +414,7 @@ static bitfield opcode_modifiers[] =
   BITFIELD (Jump),
   BITFIELD (FloatMF),
   BITFIELD (Size),
-  BITFIELD (CheckRegSize),
+  BITFIELD (CheckOperandSize),
   BITFIELD (OperandConstraint),
   BITFIELD (MnemonicSize),
   BITFIELD (No_bSuf),
@@ -864,32 +642,6 @@ next_field (char *str, char sep, char **next, char *last)
 
 static void set_bitfield (char *, bitfield *, int, unsigned int, int);
 
-static int
-set_bitfield_from_cpu_flag_init (char *f, bitfield *array, unsigned int size,
-				 int lineno)
-{
-  char *str, *next, *last;
-  unsigned int i;
-
-  for (i = 0; i < ARRAY_SIZE (cpu_flag_init); i++)
-    if (strcmp (cpu_flag_init[i].name, f) == 0)
-      {
-	/* Turn on selective bits.  */
-	char *init = xstrdup (cpu_flag_init[i].init);
-	last = init + strlen (init);
-	for (next = init; next && next < last; )
-	  {
-	    str = next_field (next, '|', &next, last);
-	    if (str)
-	      set_bitfield (str, array, 1, size, lineno);
-	  }
-	free (init);
-	return 0;
-      }
-
-  return -1;
-}
-
 static void
 set_bitfield (char *f, bitfield *array, int value,
 	      unsigned int size, int lineno)
@@ -930,16 +682,79 @@ set_bitfield (char *f, bitfield *array, int value,
 	}
     }
 
-  /* Handle CPU_XXX_FLAGS.  */
-  if (value == 1 && !set_bitfield_from_cpu_flag_init (f, array, size, lineno))
-    return;
-
   if (lineno != -1)
     fail (_("%s: %d: unknown bitfield: %s\n"), filename, lineno, f);
   else
     fail (_("unknown bitfield: %s\n"), f);
 }
 
+static void
+add_isa_dependencies (bitfield *flags, const char *f, int value,
+		      unsigned int reverse)
+{
+  unsigned int i;
+  char *str = NULL;
+  const char *isa = f;
+  bool is_isa = false, is_avx = false;
+
+  /* Need to find base entry for references to auxiliary ones.  */
+  if (strchr (f, ':'))
+    {
+      str = xstrdup (f);
+      *strchr (str, ':') = '\0';
+      isa = str;
+    }
+  for (i = 0; i < Cpu64; ++i)
+    if (strcasecmp (flags[i].name, isa) == 0)
+      {
+	flags[i].value = value;
+	if (reverse < ARRAY_SIZE (isa_reverse_deps[0])
+	    /* Don't record the feature itself here.  */
+	    && reverse != i
+	    /* Don't record base architectures.  */
+	    && reverse > Cpu686)
+	  isa_reverse_deps[i][reverse] = 1;
+	is_isa = true;
+	if (i == CpuAVX || i == CpuXOP)
+	  is_avx = true;
+	break;
+      }
+  free (str);
+
+  /* Do not turn off dependencies.  */
+  if (is_isa && !value)
+    return;
+
+  for (i = 0; i < ARRAY_SIZE (isa_dependencies); ++i)
+    if (strcasecmp (isa_dependencies[i].name, f) == 0)
+      {
+	char *deps = xstrdup (isa_dependencies[i].deps);
+	char *next = deps;
+	char *last = deps + strlen (deps);
+
+	for (; next && next < last; )
+	  {
+	    char *str = next_field (next, '|', &next, last);
+
+	    /* No AVX/XOP -> SSE reverse dependencies.  */
+	    if (is_avx && strncmp (str, "SSE", 3) == 0)
+	      add_isa_dependencies (flags, str, value, CpuMax);
+	    else
+	      add_isa_dependencies (flags, str, value, reverse);
+	  }
+	free (deps);
+
+	/* ISA extensions with dependencies need CPU_ANY_*_FLAGS emitted.  */
+	if (reverse < ARRAY_SIZE (isa_reverse_deps[0]))
+	  isa_reverse_deps[reverse][reverse] = 1;
+
+	return;
+      }
+
+  if (!is_isa)
+    fail (_("unknown bitfield: %s\n"), f);
+}
+
 static void
 output_cpu_flags (FILE *table, bitfield *flags, unsigned int size,
 		  int macro, const char *comma, const char *indent)
@@ -972,18 +787,27 @@ output_cpu_flags (FILE *table, bitfield *flags, unsigned int size,
 }
 
 static void
-process_i386_cpu_flag (FILE *table, char *flag, int macro,
+process_i386_cpu_flag (FILE *table, char *flag,
+		       const char *name,
 		       const char *comma, const char *indent,
-		       int lineno)
+		       int lineno, unsigned int reverse)
 {
   char *str, *next = flag, *last;
   unsigned int i;
   int value = 1;
+  bool is_isa = false;
   bitfield flags [ARRAY_SIZE (cpu_flags)];
 
   /* Copy the default cpu flags.  */
   memcpy (flags, cpu_flags, sizeof (cpu_flags));
 
+  if (flag == NULL)
+    {
+      for (i = 0; i < ARRAY_SIZE (isa_reverse_deps[0]); ++i)
+	flags[i].value = isa_reverse_deps[reverse][i];
+      goto output;
+    }
+
   if (flag[0] == '~')
     {
       last = flag + strlen (flag);
@@ -1010,19 +834,54 @@ process_i386_cpu_flag (FILE *table, char *flag, int macro,
       value = 0;
     }
 
+  if (name != NULL && value != 0)
+    {
+      for (i = 0; i < ARRAY_SIZE (flags); i++)
+	if (strcasecmp (flags[i].name, name) == 0)
+	  {
+	    add_isa_dependencies (flags, name, 1, reverse);
+	    is_isa = true;
+	    break;
+	  }
+    }
+
   if (strcmp (flag, "0"))
     {
+      if (is_isa)
+	return;
+
       /* Turn on/off selective bits.  */
       last = flag + strlen (flag);
       for (; next && next < last; )
 	{
 	  str = next_field (next, '|', &next, last);
-	  if (str)
+	  if (name == NULL)
 	    set_bitfield (str, flags, value, ARRAY_SIZE (flags), lineno);
+	  else
+	    add_isa_dependencies (flags, str, value, reverse);
 	}
     }
 
-  output_cpu_flags (table, flags, ARRAY_SIZE (flags), macro,
+ output:
+  if (name != NULL)
+    {
+      size_t len = strlen (name);
+      char *upper = xmalloc (len + 1);
+
+      for (i = 0; i < len; ++i)
+	{
+	  /* Don't emit #define-s for auxiliary entries.  */
+	  if (name[i] == ':')
+	    return;
+	  upper[i] = TOUPPER (name[i]);
+	}
+      upper[i] = '\0';
+      fprintf (table, "\n#define CPU_%s%s_FLAGS \\\n",
+	       flag != NULL ? "": "ANY_", upper);
+      free (upper);
+    }
+
+  output_cpu_flags (table, flags, ARRAY_SIZE (flags), name != NULL,
 		    comma, indent);
 }
 
@@ -1299,8 +1158,10 @@ output_i386_opcode (FILE *table, const char *name, char *str,
   /* Find base_opcode.  */
   base_opcode = next_field (str, ',', &str, last);
 
-  /* Find extension_opcode.  */
-  extension_opcode = next_field (str, ',', &str, last);
+  /* Find extension_opcode, if any.  */
+  extension_opcode = strchr (base_opcode, '/');
+  if (extension_opcode)
+    *extension_opcode++ = '\0';
 
   /* Find cpu_flags.  */
   cpu_flags = next_field (str, ',', &str, last);
@@ -1385,12 +1246,13 @@ output_i386_opcode (FILE *table, const char *name, char *str,
 	  filename, lineno, name, 2 * length, opcode);
 
   fprintf (table, "  { \"%s\", 0x%0*llx%s, %lu, %s,\n",
-	   name, 2 * (int)length, opcode, end, i, extension_opcode);
+	   name, 2 * (int)length, opcode, end, i,
+	   extension_opcode ? extension_opcode : "None");
 
   process_i386_opcode_modifier (table, opcode_modifier, space, prefix,
 				operand_types, lineno);
 
-  process_i386_cpu_flag (table, cpu_flags, 0, ",", "    ", lineno);
+  process_i386_cpu_flag (table, cpu_flags, NULL, ",", "    ", lineno, CpuMax);
 
   fprintf (table, "    { ");
 
@@ -1703,7 +1565,7 @@ process_i386_opcodes (FILE *table)
 {
   FILE *fp;
   char buf[2048];
-  unsigned int i, j;
+  unsigned int i, j, nr;
   char *str, *p, *last, *name;
   htab_t opcode_hash_table;
   struct opcode_hash_entry **opcode_array = NULL;
@@ -1718,7 +1580,7 @@ process_i386_opcodes (FILE *table)
 					 xcalloc, free);
 
   fprintf (table, "\n/* i386 opcode table.  */\n\n");
-  fprintf (table, "const insn_template i386_optab[] =\n{\n");
+  fprintf (table, "static const insn_template i386_optab[] =\n{\n");
 
   /* Put everything on opcode array.  */
   while (!feof (fp))
@@ -1818,15 +1680,25 @@ process_i386_opcodes (FILE *table)
 
   fclose (fp);
 
-  fprintf (table, "  { NULL, 0, 0, 0,\n");
+  fprintf (table, "};\n");
 
-  process_i386_opcode_modifier (table, "0", 0, 0, NULL, -1);
+  /* Generate opcode sets array.  */
+  fprintf (table, "\n/* i386 opcode sets table.  */\n\n");
+  fprintf (table, "static const insn_template *const i386_op_sets[] =\n{\n");
+  fprintf (table, "  i386_optab,\n");
 
-  process_i386_cpu_flag (table, "0", 0, ",", "    ", -1);
+  for (nr = j = 0; j < i; j++)
+    {
+      struct opcode_hash_entry *next = opcode_array[j];
 
-  fprintf (table, "    { ");
-  process_i386_operand_type (table, "0", stage_opcodes, "\t  ", -1);
-  fprintf (table, " } }\n");
+      do
+	{
+	  ++nr;
+	  next = next->next;
+	}
+      while (next);
+      fprintf (table, "  i386_optab + %u,\n", nr);
+    }
 
   fprintf (table, "};\n");
 }
@@ -1848,7 +1720,7 @@ process_i386_registers (FILE *table)
 	  xstrerror (errno));
 
   fprintf (table, "\n/* i386 register table.  */\n\n");
-  fprintf (table, "const reg_entry i386_regtab[] =\n{\n");
+  fprintf (table, "static const reg_entry i386_regtab[] =\n{\n");
 
   while (!feof (fp))
     {
@@ -1911,7 +1783,7 @@ process_i386_registers (FILE *table)
 
   fprintf (table, "};\n");
 
-  fprintf (table, "\nconst unsigned int i386_regtab_size = ARRAY_SIZE (i386_regtab);\n");
+  fprintf (table, "\nstatic const unsigned int i386_regtab_size = ARRAY_SIZE (i386_regtab);\n");
 }
 
 static void
@@ -1919,7 +1791,6 @@ process_i386_initializers (void)
 {
   unsigned int i;
   FILE *fp = fopen ("i386-init.h", "w");
-  char *init;
 
   if (fp == NULL)
     fail (_("can't create i386-init.h, errno = %s\n"),
@@ -1927,12 +1798,44 @@ process_i386_initializers (void)
 
   process_copyright (fp);
 
-  for (i = 0; i < ARRAY_SIZE (cpu_flag_init); i++)
+  for (i = 0; i < Cpu64; i++)
+    process_i386_cpu_flag (fp, "0", cpu_flags[i].name, "", "  ", -1, i);
+
+  for (i = 0; i < ARRAY_SIZE (isa_dependencies); i++)
     {
-      fprintf (fp, "\n#define %s \\\n", cpu_flag_init[i].name);
-      init = xstrdup (cpu_flag_init[i].init);
-      process_i386_cpu_flag (fp, init, 1, "", "  ", -1);
-      free (init);
+      char *deps = xstrdup (isa_dependencies[i].deps);
+
+      process_i386_cpu_flag (fp, deps, isa_dependencies[i].name,
+			     "", "  ", -1, CpuMax);
+      free (deps);
+    }
+
+  /* Early x87 is somewhat special: Both 287 and 387 not only add new insns
+     but also remove some.  Hence 8087 isn't a prereq to 287, and 287 isn't
+     one to 387.  We want the reverse to be true though: Disabling 8087 also
+     is to disable 287+ and later; disabling 287 also means disabling 387+.  */
+  memcpy (isa_reverse_deps[Cpu287], isa_reverse_deps[Cpu387],
+          sizeof (isa_reverse_deps[0]));
+  isa_reverse_deps[Cpu287][Cpu387] = 1;
+  memcpy (isa_reverse_deps[Cpu8087], isa_reverse_deps[Cpu287],
+          sizeof (isa_reverse_deps[0]));
+  isa_reverse_deps[Cpu8087][Cpu287] = 1;
+
+  /* While we treat POPCNT as a prereq to SSE4.2, its disabling should not
+     lead to disabling of anything else.  */
+  memset (isa_reverse_deps[CpuPOPCNT], 0, sizeof (isa_reverse_deps[0]));
+
+  for (i = Cpu686 + 1; i < ARRAY_SIZE (isa_reverse_deps); i++)
+    {
+      size_t len;
+      char *upper;
+
+      if (memchr(isa_reverse_deps[i], 1,
+	  ARRAY_SIZE (isa_reverse_deps[0])) == NULL)
+	continue;
+
+      isa_reverse_deps[i][i] = 1;
+      process_i386_cpu_flag (fp, NULL, cpu_flags[i].name, "", "  ", -1, i);
     }
 
   fprintf (fp, "\n");
